"use strict";

var moment = require('moment');

module.exports = function(sequelize, DataTypes) {
    var ASN = sequelize.define('ASN', {
            id: {
                type: DataTypes.INTEGER,
                field: 'id',
                primaryKey: true,
                autoIncrement: true
            },
            asnId: {
                type: DataTypes.STRING,
                field: 'asn_id'
            },
            asnDate: {
                type: DataTypes.DATE,
                field: 'asn_date',
                set: function (val) {
                    if (val == null) {
                        this.setDataValue('asnDate', new Date());
                    } else {
                        try {
                            var m = moment(val, "YYYY-MM-DDTHH:mm:ss.SSSZ");
                            if (!m.isValid()) {
                                m = moment(val, "YYYY-MM-DD HH:mm:ss.SSS");
                                if (!m.isValid()) {
                                    var mom = moment();
                                    logger.warn("Invalid Timestamp: " + val);
                                    this.setDataValue('asnDate', mom);
                                } else {
                                    this.setDataValue('asnDate', m)
                                }
                            } else {
                                this.setDataValue('asnDate', m)
                            }
                        } catch (e) {
                            var mo = moment();
                            this.setDataValue('asnDate', mo);
                        }
                    }
                }
            },
            poNumber: {
                type: DataTypes.STRING,
                field: 'po_number'
            },
            poDate: {
                type: DataTypes.DATE,
                field: 'po_date',
                set: function (val) {
                    if (val == null) {
                        this.setDataValue('poDate', null);
                    } else {
                        try {
                            var m = moment(val, "YYYY-MM-DDTHH:mm:ss.SSSZ");
                            if (!m.isValid()) {
                                m = moment(val, "YYYY-MM-DD HH:mm:ss.SSS");
                                if (!m.isValid()) {
                                    var mom = moment();
                                    //logger.warn("Invalid Timestamp: " + val);
                                    this.setDataValue('poDate', mom);
                                } else {
                                    this.setDataValue('poDate', m)
                                }
                            } else {
                                this.setDataValue('poDate', m)
                            }
                        } catch (e) {
                            var mo = moment();
                            this.setDataValue('poDate', mo);
                        }
                    }
                }
            },
            vendorId: {
                type: DataTypes.STRING,
                field: 'vendor_id'
            },
            vendorName: {
                type: DataTypes.STRING,
                field: 'vendor_name'
            },
            clinicId: {
                type: DataTypes.STRING,
                field: 'clinic_id'
            },
            shipToAccount: {
                type: DataTypes.STRING,
                field: 'ship_to_account'
            },
            shipToName: {
                type: DataTypes.STRING,
                field: 'ship_to_name'
            },
            shipFromId: {
                type: DataTypes.STRING,
                field: 'ship_from_id'
            },
            shipVia: {
                type: DataTypes.STRING,
                field: 'ship_via'
            },
            lineNumber: {
                type: DataTypes.STRING,
                field: 'line_number'
            },
            vendorItemId: {
                type: DataTypes.STRING,
                field: 'vendor_material'
            },
            ndc: {
                type: DataTypes.STRING,
                field: 'ndc'
            },
            sapMaterialId: {
                type: DataTypes.STRING,
                field: 'sap_material_id'
            },
            serializationNumber: {
                type: DataTypes.STRING,
                field: 'serialization_number'
            },
            shipQty: {
                type: DataTypes.INTEGER,
                field: 'ship_qty'
            },
            unitPrice: {
                type: DataTypes.DOUBLE,
                field: 'unit_price'
            },
            uom: {
                type: DataTypes.STRING,
                field: 'uom'
            },
            description: {
                type: DataTypes.STRING,
                field: 'material_description'
            },
            lot: {
                type: DataTypes.STRING,
                field: 'lot'
            },
            expiration: {
                type: DataTypes.DATE,
                field: 'expiration',
                set: function (val) {
                    if (val == null) {
                        this.setDataValue('expiration', null);
                    } else {
                        try {
                            var m = moment(val, "YYYY-MM-DDTHH:mm:ss.SSSZ");
                            if (!m.isValid()) {
                                m = moment(val, "YYYY-MM-DD HH:mm:ss.SSS");
                                if (!m.isValid()) {
                                    var mom = moment();
                                    //logger.warn("Invalid expiration Timestamp: " + val);
                                    this.setDataValue('expiration', mom);
                                } else {
                                    this.setDataValue('expiration', m)
                                }
                            } else {
                                this.setDataValue('expiration', m)
                            }
                        } catch (e) {
                            var mo = moment();
                            this.setDataValue('expiration', mo);
                        }
                    }
                }
            },
            invoiceNumber: {
                type: DataTypes.STRING,
                field: 'invoice_number'
            },
            trackingId: {
                type: DataTypes.STRING,
                field: 'tracking_id'
            },
            receivedQty: {
                type: DataTypes.INTEGER,
                field: 'received_qty',
                defaultValue: -1
            },
            lostQty: {
                type: DataTypes.INTEGER,
                field: 'lost_qty',
                defaultValue: 0
            },
            status: {
                type: DataTypes.VIRTUAL,
                get: function() {
                    var out = "created";
                    var recvd = this.get('receivedQty');
                    if (recvd) {
                        if (recvd > 0) {
                            var shipped = this.get('shipQty');
                            if (recvd < shipped) {
                                out = "partial receipt";
                            } else {
                                out = "received";
                            }
                        }
                    }
                    return out;
                }
            }
        },
        {
            timestamps: false,
            tableName: 'tbl_asn',
            indexes: [
                // Create a unique index on device timestamp
                {
                    unique: true,
                    fields: ['asnId','poNumber', 'lineNumber', 'vendorItemId' ]
                },
            ]
        }
    );
    var AsnAddress = require("./asnAddress")(sequelize, DataTypes);
    ASN.hasMany(AsnAddress, {as: 'Addresses', foreignKey: 'asnId'});
    return ASN;
};
